------------------------------------------------------------------------------
--  Test: Access_Control_002
--  Verification: This test verifies permission check is not only performed 
--      on the level of an object but also considering its parent objects
--
--  Test Details:
--      1.  An account will be created and granted select on a table
--      2.  Select access will then be verified
--      3.  Select permission on the table will be revoked
--      4.  Select permission on the table will be granted on the table schema
--      5.  Select access will then be verified
--
--  Test Dependenices:
--      1.  This test must be executed as a sysadmin
------------------------------------------------------------------------------

DECLARE @error_message 	nvarchar(1024)

BEGIN TRY
	SET NOCOUNT ON

	------------------------------------------------------------------------------
	-- If the version is not SP2 then do not run the script
	------------------------------------------------------------------------------
	IF SERVERPROPERTY( N'ProductVersion' ) <> N'10.0.4000.0'
	BEGIN
		SET @error_message = N'TESTRESULT: FAIL [Access_Control_002] This test can only be executed on SQL Server 2008 SP2';
		PRINT @error_message
		RETURN;
	END

	------------------------------------------------------------------------------
	-- Verify test dependencies
	------------------------------------------------------------------------------
	IF ( IS_SRVROLEMEMBER( N'sysadmin' ) = 0 )
	BEGIN
		SET @error_message = N'TESTRESULT: FAIL [Access_Control_002] This test must be executed by a sysadmin';
		PRINT @error_message
		RETURN;
	END

	------------------------------------------------------------------------------
	-- Create account and test objects
	------------------------------------------------------------------------------
	CREATE USER [USR_Access_Control_002] WITHOUT LOGIN;
	EXEC ( N'CREATE SCHEMA [SCH_Access_Control_002]' );
	CREATE TABLE [SCH_Access_Control_002].[TBL_Access_Control_002] ( c1 int );
	INSERT [SCH_Access_Control_002].[TBL_Access_Control_002] VALUES ( 1 );

	------------------------------------------------------------------------------
	-- Grant Select permission on the table to the account
	------------------------------------------------------------------------------
	GRANT SELECT ON [SCH_Access_Control_002].[TBL_Access_Control_002] TO [USR_Access_Control_002];

	------------------------------------------------------------------------------
	-- Verify access to the object is denied
	------------------------------------------------------------------------------
	BEGIN TRY
		EXEC( N'SELECT * FROM [SCH_Access_Control_002].[TBL_Access_Control_002]' ) AS USER = N'USR_Access_Control_002';
	END TRY
	BEGIN CATCH
		--------------------------------------------------------------------------------------------------------
		-- Expect No Error
		--------------------------------------------------------------------------------------------------------
		SET @error_message = N'TESTRESULT: FAIL [Access_Control_002] Expect no error when selecting with select permission on the table';
		PRINT @error_message
		RETURN;
	END CATCH

	------------------------------------------------------------------------------
	-- Revoke Select permission on the table to the account
	-- Grant Select permission on the table schema
	------------------------------------------------------------------------------
	REVOKE SELECT ON [SCH_Access_Control_002].[TBL_Access_Control_002] TO [USR_Access_Control_002];
	GRANT SELECT ON SCHEMA::[SCH_Access_Control_002] TO [USR_Access_Control_002];

	------------------------------------------------------------------------------
	-- Verify access to the object is allowed
	------------------------------------------------------------------------------
	BEGIN TRY
		EXEC( N'SELECT * FROM [SCH_Access_Control_002].[TBL_Access_Control_002]' ) AS USER = N'USR_Access_Control_002';
	END TRY
	BEGIN CATCH
		--------------------------------------------------------------------------------------------------------
		-- Expect No Error
		--------------------------------------------------------------------------------------------------------
		SET @error_message = N'TESTRESULT: FAIL [Access_Control_002] Expect no error when selecting with select permission on the schema';
		PRINT @error_message
		RETURN;
	END CATCH

	PRINT 'TESTRESULT: PASS [Access_Control_002]';
END TRY
BEGIN CATCH

	SET @error_message = N'TESTRESULT: FAIL [Access_Control_002] ' + error_message();
	PRINT @error_message

END CATCH
go

--------------------------------------------------------------------------------------------------------
-- Cleanup ALL accounts and objects
--------------------------------------------------------------------------------------------------------
IF ( OBJECT_ID( N'SCH_Access_Control_002.TBL_Access_Control_002' ) IS NOT NULL )
BEGIN
	DROP TABLE [SCH_Access_Control_002].[TBL_Access_Control_002];
END
go

IF ( SCHEMA_ID( N'SCH_Access_Control_002' ) IS NOT NULL )
BEGIN
	DROP SCHEMA [SCH_Access_Control_002];
END
go

IF ( USER_ID( N'USR_Access_Control_002' ) IS NOT NULL )
BEGIN
	DROP USER [USR_Access_Control_002];
END
go
